@@ -12,6 +12,7 @@ from xarray.core.coordinates import Coordinates
 from xarray.core.dataarray import DataArray
 from xarray.core.dataset import Dataset
 from xarray.core.datatree import DataTree
+from xarray.core.datatree_mapping import map_over_datasets
 from xarray.core.formatting import diff_datatree_repr
 from xarray.core.indexes import Index, PandasIndex, PandasMultiIndex, default_indexes
 from xarray.core.variable import IndexVariable, Variable
@@ -85,14 +86,25 @@ def assert_isomorphic(a: DataTree, b: DataTree):
 
 def maybe_transpose_dims(a, b, check_dim_order: bool):
     """Helper for assert_equal/allclose/identical"""
+
     __tracebackhide__ = True
-    if not isinstance(a, Variable | DataArray | Dataset):
+
+    def _maybe_transpose_dims(a, b):
+        if not isinstance(a, Variable | DataArray | Dataset):
+            return b
+        if set(a.dims) == set(b.dims):
+            # Ensure transpose won't fail if a dimension is missing
+            # If this is the case, the difference will be caught by the caller
+            return b.transpose(*a.dims)
         return b
-    if not check_dim_order and set(a.dims) == set(b.dims):
-        # Ensure transpose won't fail if a dimension is missing
-        # If this is the case, the difference will be caught by the caller
-        return b.transpose(*a.dims)
-    return b
+
+    if check_dim_order:
+        return b
+
+    if isinstance(a, DataTree):
+        return map_over_datasets(_maybe_transpose_dims, a, b)
+
+    return _maybe_transpose_dims(a, b)
 
 
 @ensure_warnings
@@ -227,6 +239,11 @@ def assert_allclose(
         b = getattr(b, "variable", b)
         return a.dims == b.dims and (a._data is b._data or equiv(a.data, b.data))
 
+    def compat_node(a, b):
+        return a.ds._coord_names == b.ds._coord_names and utils.dict_equiv(
+            a.variables, b.variables, compat=compat_variable
+        )
+
     if isinstance(a, Variable):
         allclose = compat_variable(a, b)
         assert allclose, formatting.diff_array_repr(a, b, compat=equiv)
@@ -243,6 +260,11 @@ def assert_allclose(
     elif isinstance(a, Coordinates):
         allclose = utils.dict_equiv(a.variables, b.variables, compat=compat_variable)
         assert allclose, formatting.diff_coords_repr(a, b, compat=equiv)
+    elif isinstance(a, DataTree):
+        allclose = utils.dict_equiv(
+            dict(a.subtree_with_keys), dict(b.subtree_with_keys), compat=compat_node
+        )
+        assert allclose, formatting.diff_datatree_repr(a, b, compat=equiv)
     else:
         raise TypeError(f"{type(a)} not supported by assertion comparison")
 
