@@ -5,6 +5,7 @@ import pickle
 import sys
 from contextlib import suppress
 from textwrap import dedent
+from typing import Any
 
 import numpy as np
 import pandas as pd
@@ -94,13 +95,14 @@ class TestVariable(DaskTestCase):
         self.assertLazyAndIdentical(self.eager_var, self.lazy_var.copy(deep=True))
 
     def test_chunk(self):
-        for chunks, expected in [
+        test_cases: list[tuple[int | dict[str, Any], tuple[tuple[int, ...], ...]]] = [
             ({}, ((2, 2), (2, 2, 2))),
             (3, ((3, 1), (3, 3))),
             ({"x": 3, "y": 3}, ((3, 1), (3, 3))),
             ({"x": 3}, ((3, 1), (2, 2, 2))),
             ({"x": (3, 1)}, ((3, 1), (2, 2, 2))),
-        ]:
+        ]
+        for chunks, expected in test_cases:
             rechunked = self.lazy_var.chunk(chunks)
             assert rechunked.chunks == expected
             self.assertLazyAndIdentical(self.eager_var, rechunked)
@@ -258,7 +260,7 @@ class TestVariable(DaskTestCase):
         with pytest.raises(NotImplementedError, match="dask"):
             v.argsort()
         with pytest.raises(NotImplementedError, match="dask"):
-            v[0].item()
+            v[0].item()  # type: ignore[attr-defined]
 
     def test_univariate_ufunc(self):
         u = self.eager_var
@@ -298,7 +300,7 @@ class TestVariable(DaskTestCase):
 
         (v2,) = dask.persist(v)
         assert v is not v2
-        assert len(v2.__dask_graph__()) < len(v.__dask_graph__())
+        assert len(v2.__dask_graph__()) < len(v.__dask_graph__())  # type: ignore[arg-type]
         assert v2.__dask_keys__() == v.__dask_keys__()
         assert dask.is_dask_collection(v)
         assert dask.is_dask_collection(v2)
@@ -345,7 +347,9 @@ class TestDataArrayAndDataset(DaskTestCase):
         )
 
     def test_chunk(self) -> None:
-        for chunks, expected in [
+        test_cases: list[
+            tuple[int | str | dict[str, Any], tuple[tuple[int, ...], ...]]
+        ] = [
             ({}, ((2, 2), (2, 2, 2))),
             (3, ((3, 1), (3, 3))),
             ({"x": 3, "y": 3}, ((3, 1), (3, 3))),
@@ -354,7 +358,8 @@ class TestDataArrayAndDataset(DaskTestCase):
             ({"x": "16B"}, ((1, 1, 1, 1), (2, 2, 2))),
             ("16B", ((1, 1, 1, 1), (1,) * 6)),
             ("16MB", ((4,), (6,))),
-        ]:
+        ]
+        for chunks, expected in test_cases:
             # Test DataArray
             rechunked = self.lazy_array.chunk(chunks)
             assert rechunked.chunks == expected
@@ -367,7 +372,7 @@ class TestDataArrayAndDataset(DaskTestCase):
             lazy_dataset = self.lazy_array.to_dataset()
             eager_dataset = self.eager_array.to_dataset()
             expected_chunksizes = dict(zip(lazy_dataset.dims, expected, strict=True))
-            rechunked = lazy_dataset.chunk(chunks)
+            rechunked = lazy_dataset.chunk(chunks)  # type: ignore[assignment]
 
             # Dataset.chunks has a different return type to DataArray.chunks - see issue #5843
             assert rechunked.chunks == expected_chunksizes
@@ -442,7 +447,11 @@ class TestDataArrayAndDataset(DaskTestCase):
 
         assert kernel_call_count == 0
         out = xr.concat(
-            [ds1, ds2, ds3], dim="n", data_vars="different", coords="different"
+            [ds1, ds2, ds3],
+            dim="n",
+            data_vars="different",
+            coords="different",
+            compat="equals",
         )
         # each kernel is computed exactly once
         assert kernel_call_count == 6
@@ -484,7 +493,11 @@ class TestDataArrayAndDataset(DaskTestCase):
         # stop computing variables as it would not have any benefit
         ds4 = Dataset(data_vars={"d": ("x", [2.0])}, coords={"c": ("x", [2.0])})
         out = xr.concat(
-            [ds1, ds2, ds4, ds3], dim="n", data_vars="different", coords="different"
+            [ds1, ds2, ds4, ds3],
+            dim="n",
+            data_vars="different",
+            coords="different",
+            compat="equals",
         )
         # the variables of ds1 and ds2 were computed, but those of ds3 didn't
         assert kernel_call_count == 22
@@ -505,7 +518,11 @@ class TestDataArrayAndDataset(DaskTestCase):
 
         # now check that concat() is correctly using dask name equality to skip loads
         out = xr.concat(
-            [ds1, ds1, ds1], dim="n", data_vars="different", coords="different"
+            [ds1, ds1, ds1],
+            dim="n",
+            data_vars="different",
+            coords="different",
+            compat="equals",
         )
         assert kernel_call_count == 24
         # variables are not loaded in the output
@@ -589,11 +606,12 @@ class TestDataArrayAndDataset(DaskTestCase):
         u = self.eager_array.assign_coords(y=range(6))
         v = self.lazy_array.assign_coords(y=range(6))
 
-        for kwargs in [
+        kwargs_list: list[dict[str, Any]] = [
             {"x": [2, 3, 4]},
             {"x": [1, 100, 2, 101, 3]},
             {"x": [2.5, 3, 3.5], "y": [2, 2.5, 3]},
-        ]:
+        ]
+        for kwargs in kwargs_list:
             expected = u.reindex(**kwargs)
             actual = v.reindex(**kwargs)
             self.assertLazyAndAllClose(expected, actual)
@@ -654,7 +672,9 @@ class TestDataArrayAndDataset(DaskTestCase):
         data = da.random.normal(size=(2, 3, 4), chunks=(1, 3, 4))
         arr = DataArray(data, dims=("w", "x", "y"))
         stacked = arr.stack(z=("x", "y"))
-        z = pd.MultiIndex.from_product([np.arange(3), np.arange(4)], names=["x", "y"])
+        z = pd.MultiIndex.from_product(
+            [list(range(3)), list(range(4))], names=["x", "y"]
+        )
         expected = DataArray(data.reshape(2, -1), {"z": z}, dims=["w", "z"])
         assert stacked.data.chunks == expected.data.chunks
         self.assertLazyAndEqual(expected, stacked)
@@ -1141,6 +1161,21 @@ def test_auto_chunk_da(obj):
     assert actual.chunks == expected.chunks
 
 
+def test_auto_chunk_da_cftime():
+    yrs = np.arange(2000, 2120)
+    cftime_dates = xr.date_range(
+        start=f"{yrs[0]}-01-01", end=f"{yrs[-1]}-12-31", freq="1YE", use_cftime=True
+    )
+    yr_array = np.tile(cftime_dates.values, (10, 1))
+    da = xr.DataArray(
+        yr_array, dims=["x", "t"], coords={"x": np.arange(10), "t": cftime_dates}
+    ).chunk({"x": 4, "t": 5})
+    actual = da.chunk("auto").data
+    expected = da.data.rechunk({0: 10, 1: 120})
+    np.testing.assert_array_equal(actual, expected)
+    assert actual.chunks == expected.chunks
+
+
 def test_map_blocks_error(map_da, map_ds):
     def bad_func(darray):
         return (darray * darray.x + 5 * darray.y)[:1, :1]
@@ -1155,10 +1190,10 @@ def test_map_blocks_error(map_da, map_ds):
         xr.map_blocks(returns_numpy, map_da)
 
     with pytest.raises(TypeError, match=r"args must be"):
-        xr.map_blocks(operator.add, map_da, args=10)
+        xr.map_blocks(operator.add, map_da, args=10)  # type: ignore[arg-type]
 
     with pytest.raises(TypeError, match=r"kwargs must be"):
-        xr.map_blocks(operator.add, map_da, args=[10], kwargs=[20])
+        xr.map_blocks(operator.add, map_da, args=[10], kwargs=[20])  # type: ignore[arg-type]
 
     def really_bad_func(darray):
         raise ValueError("couldn't do anything.")
@@ -1366,7 +1401,7 @@ def test_map_blocks_ds_transformations(func, map_ds):
 def test_map_blocks_da_ds_with_template(obj):
     func = lambda x: x.isel(x=[1])
     # a simple .isel(x=[1, 5, 9]) puts all those in a single chunk.
-    template = xr.concat([obj.isel(x=[i]) for i in [1, 5, 9]], dim="x")
+    template = xr.concat([obj.isel(x=[i]) for i in [1, 5, 9]], data_vars=None, dim="x")
     with raise_if_dask_computes():
         actual = xr.map_blocks(func, obj, template=template)
     assert_identical(actual, template)
@@ -1430,7 +1465,7 @@ def test_map_blocks_errors_bad_template(obj):
     with pytest.raises(ValueError, match=r"Received dimension 'x' of length 1"):
         xr.map_blocks(lambda x: x.isel(x=[1]), obj, template=obj).compute()
     with pytest.raises(TypeError, match=r"must be a DataArray"):
-        xr.map_blocks(lambda x: x.isel(x=[1]), obj, template=(obj,)).compute()
+        xr.map_blocks(lambda x: x.isel(x=[1]), obj, template=(obj,)).compute()  # type: ignore[arg-type]
     with pytest.raises(ValueError, match=r"map_blocks requires that one block"):
         xr.map_blocks(
             lambda x: x.isel(x=[1]).assign_coords(x=10), obj, template=obj.isel(x=[1])
@@ -1439,7 +1474,9 @@ def test_map_blocks_errors_bad_template(obj):
         xr.map_blocks(
             lambda a: a.isel(x=[1]).assign_coords(x=[120]),  # assign bad index values
             obj,
-            template=xr.concat([obj.isel(x=[i]) for i in [1, 5, 9]], dim="x"),
+            template=xr.concat(
+                [obj.isel(x=[i]) for i in [1, 5, 9]], data_vars=None, dim="x"
+            ),
         ).compute()
 
 
@@ -1764,10 +1801,10 @@ def test_graph_manipulation():
     for a, b in ((v, v2), (da, da2), (ds, ds2)):
         assert a.__dask_layers__() != b.__dask_layers__()
         assert len(a.__dask_layers__()) == len(b.__dask_layers__())
-        assert a.__dask_graph__().keys() != b.__dask_graph__().keys()
-        assert len(a.__dask_graph__()) == len(b.__dask_graph__())
-        assert a.__dask_graph__().layers.keys() != b.__dask_graph__().layers.keys()
-        assert len(a.__dask_graph__().layers) == len(b.__dask_graph__().layers)
+        assert a.__dask_graph__().keys() != b.__dask_graph__().keys()  # type: ignore[union-attr]
+        assert len(a.__dask_graph__()) == len(b.__dask_graph__())  # type: ignore[arg-type]
+        assert a.__dask_graph__().layers.keys() != b.__dask_graph__().layers.keys()  # type: ignore[union-attr]
+        assert len(a.__dask_graph__().layers) == len(b.__dask_graph__().layers)  # type: ignore[union-attr]
 
     # Above we performed a slice operation; adding the two slices back together creates
     # a diamond-shaped dependency graph, which in turn will trigger a collision in layer
